from os.path import exists
from os import getpid
from ConfigParser import ConfigParser

from Thump.lib.Configuration import filenames

## Super generic ##

class File(file):
  """Generic file object with convenient data access methods"""
  
  def __init__(self, filename, mode="rw"):
    if not exists(filename): open(filename, "w").close()
    file.__init__(self, filename, mode)

  def reOpen(self, mode=None):
    self.close()
    file.__init__(self.name, mode and mode or self.mode)

  def niceWriteLines(self, lines):
    return self.writelines( [ line.rstrip()+"\n" for line in lines ] )

  def readStrippedLines(self):
    return [ line.strip() for line in self.readlines() ]
  
  def readStrippedLine(self):
    return self.readline().strip()

  def tail(self, numLines=1):
    #mylines = list()
    #for line in self.readStrippedLines():
      #if len(line):
        #mylines.append(line)
        #if len(mylines) > lines:
          #del mylines[0]

    mylines = self.readStrippedLines()

    return mylines[-numLines:]
    #return mylines

class NullFile:
  close=read=readline=readlines=seek=writelines=write=lambda *a,**b:None

## less generic ##

class PidFile(File):
  """Pid file, should contain one process id, on the first line."""

  def __init__(self, filename, mode="rw"):
    File.__init__(self, filename, mode)

  def getPid(self):
    self.seek(0)
    try:
      return int(self.readline().strip())
    except ValueError:
      return None


class MultiPidFile(File):
  """MultiPidFile is like PidFile, but contains one pid per line."""

  def __init__(self, filename, mode="rw"):
    File.__init__(self, filename, mode)

  def getPids(self):
    self.seek(0)
    pidlist = list()
    for line in self.readStrippedLines():
      try:
        pidlist.append(int(line))
      except ValueError:
        pass

    return pidlist

class ConfigFile(File):

  parser = ConfigParser()
  
  def __init__(self, filename, mode="rw"):
    File.__init__(self, filename, mode)

    self.parser.readfp(self)

  sections = parser.sections
  hasSection = parser.has_section
  hasOption = parser.has_option
  options = parser.options
  getOption = parser.get
  getInt = parser.getint
  getFloat = parser.getfloat
  getBoolean = parser.getboolean
  removeSection = parser.remove_section
  removeOption = parser.remove_option
  setOption = parser.set
  save = lambda: parser.write(self)



## specific to thump ##

class FePidFile(MultiPidFile):
  """File containing list of pid's of currently running frontends/clients"""
  def __init__(self, mode="rw"):
    MultiPidFile.__init__(self, filenames.fepidfile, mode)

class ListFile(File):
  """Playlist, contains path to a file on each line."""

  def __init__(self, mode="rw"):
    File.__init__(self, filenames.listfile, mode)

  def getSong(self):
    return self.readStrippedLine()
  def getAllSongs(self):
    return self.readStrippedLines()

class AccessFile(File):

  def __init__(self, mode="rw"):
    File.__init__(self, filenames.access_log, mode)

  def __dict_from_line(self, line):
    items = [ item.strip() for item in line.split(":@@:") ]
    return { "time": items[0], "song": items[1] }

  def logAccess(self, song):
    self.write("%s :@@: %s\n" % (time.asctime(), song))
  
  def getSong(self):
    return self.__dict_from_line(self.readStrippedLine())

  def getAllSongs(self):
    return [ self.__dict_from_line(line) for line in self.readStrippedLines() ]

class LiveFile(File):
  # not sure what to do with this quite yet

  def __init__(self, mode="rw"):
    File.__init__(self, filenames.livefile, mode)

class ErrorLog(File):

  def __init__(self, mode="a"):
    File.__init__(self, filenames.error_log, mode)
  
  def writeError(self, msg):
    self.write("%s %s (%s): %s\n" % (time.asctime(), sys.argv[0].split("/")[-1], getpid(), msg))

  def unknownCommand(self, command):
    self.writeError("Received unknown command `%s'" % command)

  def decoderHung(self):
    self.writeError("Decoder is hung")

  def die(self, msg=None):
    if msg: self.writeError(msg)
    self.writeError("Exiting")

class StderrFile(File):

  def __init__(self, mode="w"):
    File.__init__(self, filenames.stderrfile, mode)

